@@ -93,15 +93,13 @@ def _get_string(source, i):
 def _get_char(source, start, i):
     # NOTE(nnorwitz): may not be quite correct, should be good enough.
     i = source.find("'", i + 1)
-    while source[i - 1] == '\\':
+    while i != -1 and source[i - 1] == '\\':
         # Need to special case '\\'.
-        if (i - 2) > start and source[i - 2] == '\\':
+        if source[i - 2] == '\\':
             break
         i = source.find("'", i + 1)
-    # Try to handle unterminated single quotes (in a #if 0 block).
-    if i < 0:
-        i = start
-    return i + 1
+    # Try to handle unterminated single quotes.
+    return i + 1 if i != -1 else start + 1
 
 
 def get_tokens(source):
@@ -145,12 +143,10 @@ def get_tokens(source):
                 i += 1
             # String and character constants can look like a name if
             # they are something like L"".
-            if (source[i] == "'" and (i - start) == 1 and
-                    source[start:i] in 'uUL'):
-                # u, U, and L are valid C++0x character preffixes.
+            if source[i] == "'" and source[start:i] in _STR_PREFIXES:
                 token_type = CONSTANT
                 i = _get_char(source, start, i)
-            elif source[i] == "'" and source[start:i] in _STR_PREFIXES:
+            elif source[i] == '"' and source[start:i] in _STR_PREFIXES:
                 token_type = CONSTANT
                 i = _get_string(source, i)
         elif c == '/' and source[i + 1] == '/':  # Find // comments.
@@ -208,11 +204,12 @@ def get_tokens(source):
                 while source[i] in int_or_float_digits2:
                     i += 1
             # Handle integer (and float) suffixes.
-            for suffix in ('ull', 'll', 'ul', 'l', 'f', 'u'):
-                size = len(suffix)
-                if suffix == source[i:i + size].lower():
-                    i += size
-                    break
+            if source[i].isalpha():
+                for suffix in ('ull', 'll', 'ul', 'l', 'f', 'u'):
+                    size = len(suffix)
+                    if suffix == source[i:i + size].lower():
+                        i += size
+                        break
         elif c == '"':                           # Find string.
             token_type = CONSTANT
             i = _get_string(source, i)
@@ -221,49 +218,56 @@ def get_tokens(source):
             i = _get_char(source, start, i)
         elif c == '#':                           # Find pre-processor command.
             token_type = PREPROCESSOR
-            got_if = source[i:i + 3] == '#if' and source[i + 3:i + 4].isspace()
-            if got_if:
-                count_ifs += 1
-            elif source[i:i + 6] == '#endif':
+            got_if = source[i:i + 3] == '#if'
+            if ignore_errors and source[i:i + 6] == '#endif':
                 count_ifs -= 1
                 if count_ifs == 0:
                     ignore_errors = False
 
-            # TODO(nnorwitz): handle preprocessor statements (\ continuations).
+            # Handle preprocessor statements (\ continuations).
             while True:
                 i1 = source.find('\n', i)
                 i2 = source.find('//', i)
                 i3 = source.find('/*', i)
                 i4 = source.find('"', i)
-                # NOTE(nnorwitz): doesn't handle comments in #define macros.
                 # Get the first important symbol (newline, comment, EOF/end).
                 i = min([x for x in (i1, i2, i3, i4, end) if x != -1])
 
+                # Handle comments in #define macros.
+                if i == i3:
+                    i = source.find('*/', i) + 2
+                    source = source[:i3].ljust(i) + source[i:]
+                    continue
+
                 # Handle #include "dir//foo.h" properly.
                 if source[i] == '"':
                     i = source.find('"', i + 1) + 1
                     assert i > 0
                     continue
+
                 # Keep going if end of the line and the line ends with \.
-                if not (i == i1 and source[i - 1] == '\\'):
-                    if got_if:
-                        condition = source[start + 4:i].lstrip()
-                        if (condition.startswith('0') or
-                                condition.startswith('(0)')):
-                            ignore_errors = True
-                    break
-                i += 1
+                if i == i1 and source[i - 1] == '\\':
+                    i += 1
+                    continue
+
+                if got_if:
+                    condition = source[start + 4:i].lstrip()
+                    if (
+                        ignore_errors or
+                        condition.startswith('0') or
+                        condition.startswith('(0)')
+                    ):
+                        count_ifs += 1
+                        ignore_errors = True
+                break
         elif c == '\\':                          # Handle \ in code.
             # This is different from the pre-processor \ handling.
             i += 1
             continue
         elif ignore_errors:
-            # The tokenizer seems to be in pretty good shape. This
-            # raise is conditionally disabled so that bogus code
-            # in an #if 0 block can be handled. Since we will ignore
-            # it anyways, this is probably fine. So disable the
-            # exception and  return the bogus char.
+            # Ignore bogus code when we are inside an #if block.
             i += 1
+            continue
         else:
             sys.stderr.write('Got invalid token in %s @ %d token:%s: %r\n' %
                              ('?', i, c, source[i - 10:i + 10]))
